% % This script computes the marginal posterior densities of the coefficients 
% % assoicated with the covariates that appear in both the outcome and 
% % treatment equations (i.e. those partially identified)
% % The relevant covariates are: intercept, FamilyIncome, HighSchool,
% % ALevel, Degree, Union, Married
% % outputs: 
% % palpihat: marginal posterior density of alpi
% % store_Ealpi: first column contains the posterior means of alpi
% %              second column contains the posterior std dev of alpi
% %
% % See:
% % Chan, J.C.C. and Tobias, J.L. (2015). "Priors and Posterior
% % Computation in Linear Endogenous Variables Models with Imperfect
% % Instruments," Journal of Applied Econometrics, 30(4), 650-674
% %
% % =======================================================================

idx = [1 2 3 8 9 10 11 12 13];
idz = [1 8 9 2 3 4 5 6 7];  
M = size(store_theta,1);
ngrid = 500;
store_Ealpi = zeros(length(idx)-2,2);
for i=1:length(idz)-2
    store_palpi = zeros(ngrid,1);
    if i == 1
        alpgrid = linspace(-1,4,ngrid)';
    elseif i == 2
        alpgrid = linspace(-.003,.005,ngrid)';
    elseif i>=3
        alpgrid = linspace(-.2,.6,ngrid)';
    end
    store_pgam = zeros(ngrid,1);
    for j=1:M
        theta = store_theta(j,:)';
        Sig = store_Sig(j,:)';
        tempsig = Sig(2)/Sig(3);
        a = [Sig(3) theta(na+2) theta(end-1) theta(end)...
            theta(1)-tempsig*theta(na+2) theta(2)-tempsig*theta(end-1)...
            theta(3)-tempsig*theta(end) theta(na+1)+tempsig Sig(1)-Sig(2)^2/Sig(3)];
        
        sigv = sqrt(a(1));
        beta0 = a(2);
        beta1 = a(3);
        beta2 = a(4);
        a0 = a(5);
        a1 = a(6);
        a2 = a(7);
        b = a(8);
        c = sqrt(a(9));
        a3 = theta(idx([1 4:end])) - tempsig*theta(na+1+([1 idz(4:end)]));
        beta3 = theta(na+1+([1 idz(4:end)]));
        betai = sign(beta3(i))*max(.05,abs(beta3(i)));
        
        amai = alpgrid-a3(i);
        sigu2 = c^2+amai.^2*sigv^2/betai^2;
        alp0 = a0+amai*beta0/betai;
        alp1 = a1+amai*beta1/betai;
        alp2 = a2+amai*beta2/betai;       
        
        gam = b-amai/betai;
        rho2 = amai.^2*sigv^2/betai^2./sigu2;
        tempalp0 = -.5*invVtheta(1,1)*alp0.^2;
        tempalp1 = -.5*invVtheta(2,2)*alp1.^2;
        tempalp2 = -.5*invVtheta(3,3)*alp2.^2;
        tempalpi = 0;
        for ii = 1:6
            alpi = a3(1+ii)+amai*beta3(1+ii)/betai;
            tempalpi = tempalpi -.5*invVtheta(7+ii,7+ii)*alpi.^2;
        end   
        tempJ = log(sigv/abs(betai)) - .5*log(sigu2);
        detSig = (1-rho2).*sigu2*sigv^2;
        tempSig = -.5*(nu0+3)*log(detSig) -.5./detSig.*(sigv^2 + sigu2);
        if priorflag == 1; 
            palpi = exp(tempalp0+tempalp1+tempalp2+tempalpi+tempSig+tempJ);
        else        
            ind1 = abs(alp1)<=abs(gamub*beta1);
            ind2 = abs(alp2)<=abs(gamub*beta2);
            palpi = exp(tempalp0+tempalp1+tempalp2+tempalpi+tempSig+tempJ).*ind1.*ind2;
        end
        store_palpi = palpi + store_palpi;    
    end
    palpihat = store_palpi/M;
    kalpi = (alpgrid(3)-alpgrid(2))*sum(palpihat);
    palpihat = palpihat/kalpi;
    Ealpi = alpgrid'*palpihat/sum(palpihat);
    Ealpi2 = (alpgrid.^2)'*palpihat/sum(palpihat);
    alpistd = sqrt(Ealpi2-Ealpi^2);    
    store_Ealpi(i,:) = [Ealpi alpistd];
   
    figure; plot(alpgrid,palpihat); box off;
    set(gcf,'Position',[100 100 500 400]);
    switch i
        case 1 
            title('intercept');
        case 2
            title('FamilyIncome');
        case 3
            title('HighSchool');
        case 4 
            title('ALevel');
        case 5
            title('Degree');
        case 6
            title('Union');
        case 7
            title('Married');
    end
       
   
end