% % this script computes the marginal posterior densities of the coefficients 
% % assoicated with the IVs (alp1 and alp2)
% % X = [ones(N,1) mombmi dadbmi jobtenure jobtenure.^2 experience
% %     experience.^2 famincome highschool ALevel degree union married
% %     momdegree mommanprof daddegree dadmanprof];
% % Z = [ones(N,1) famincome highschool ALevel degree union married
% %     mombmi dadbmi];
% % outputs: 
% % palp1hat: marginal posterior density of alp1
% % Ealp1: posterior mean of alp1
% % alp1std: posterior std dev of alp1
% % palp2hat: marginal posterior density of alp2
% % Ealp2: posterior mean of alp2
% % alp2std: posterior std dev of alp2
% %
% % See:
% % Chan, J.C.C. and Tobias, J.L. (2015). "Priors and Posterior
% % Computation in Linear Endogenous Variables Models with Imperfect
% % Instruments," Journal of Applied Econometrics, 30(4), 650-674
% %
% % =======================================================================

M = size(store_theta,1);
ngrid = 500;
alp1grid = linspace(-2.5*delta,2.5*delta,ngrid)';  % grid for alp1
alp2grid = linspace(-2.5*delta,2.5*delta,ngrid)';  % grid for alp2

store_palp1 = zeros(ngrid,1);
for j=1:M    
    theta = store_theta(j,:)';
    Sig = store_Sig(j,:)';
    tempsig = Sig(2)/Sig(3);
    a = [Sig(3) theta(na+2) theta(end-1) theta(end)...
        theta(1)-tempsig*theta(na+2) theta(2)-tempsig*theta(end-1)...
        theta(3)-tempsig*theta(end) theta(na+1)+tempsig Sig(1)-Sig(2)^2/Sig(3)];        
    sigv = sqrt(a(1));
    beta0 = a(2);
    beta1 = a(3);
    beta2 = a(4);
    a0 = a(5);
    a1 = a(6);
    a2 = a(7);
    b = a(8);
    c = sqrt(a(9));
    betai = theta(na+1+2:na+1+7);
    ai = theta(8:13)-tempsig*betai;        
    ama1 = alp1grid-a1;
    sigu2 = c^2+ama1.^2*sigv^2/beta1^2;
    alp0 = a0+ama1*beta0/beta1;
    alp2 = a2+ama1*beta2/beta1;
    gam = b-ama1/beta1;
    rho2 = ama1.^2*sigv^2/beta1^2./sigu2;
    tempalp0 = -.5*invVtheta(1,1)*alp0.^2;
    tempalp1 = -.5*invVtheta(2,2)*alp1grid.^2;
    tempalp2 = -.5*invVtheta(3,3)*alp2.^2;   
    tempalpi = 0;
    for ii = 1:6
        alpi = ai(ii)+ama1*betai(ii)/beta1;
        tempalpi = tempalpi -.5*invVtheta(7+ii,7+ii)*alpi.^2;
    end   
    tempJ = log(sigv/abs(beta1)) - .5*log(sigu2);    
    if priorflag == 1; % see IVnonID.m for description
        detSig = (1-rho2).*sigu2*sigv^2;
        tempSig = -.5*(nu0+3)*log(detSig) -.5./detSig.*(sigv^2 + sigu2);
        palp1 = exp(tempalp0 + tempalp1 + tempalp2 + tempalpi + tempSig + tempJ);
    elseif priorflag == 2      
        ind1 = abs(alp1grid)<=abs(gamub*beta1);
        ind2 = abs(alp2)<=abs(gamub*beta2);   
        detSig = (1-rho2).*sigu2*sigv^2;
        tempSig = -.5*(nu0+3)*log(detSig) -.5./detSig.*(sigv^2 + sigu2);
        palp1 = exp(tempalp0 + tempalp1 + tempalp2 + tempalpi + tempSig + tempJ).*ind2.*ind1;        
    end
    store_palp1 = palp1 + store_palp1;  
end
palp1hat = store_palp1/M;
kalp1 = (alp1grid(3)-alp1grid(2))*sum(palp1hat);
palp1hat = palp1hat/kalp1;
Ealp1 = alp1grid'*palp1hat/sum(palp1hat);
Ealp12 = (alp1grid.^2)'*palp1hat/sum(palp1hat);
alp1std = sqrt(Ealp12-Ealp1^2);

spalp2 = zeros(ngrid,1);
for j=1:M  
    theta = store_theta(j,:)';
    Sig = store_Sig(j,:)';
    tempsig = Sig(2)/Sig(3);
    a = [Sig(3) theta(na+2) theta(end-1) theta(end)...
        theta(1)-tempsig*theta(na+2) theta(2)-tempsig*theta(end-1)...
        theta(3)-tempsig*theta(end) theta(na+1)+tempsig Sig(1)-Sig(2)^2/Sig(3)];        
    sigv = sqrt(a(1));
    beta0 = a(2);
    beta1 = a(3);
    beta2 = a(4);
    a0 = a(5);
    a1 = a(6);
    a2 = a(7);
    b = a(8);
    c = sqrt(a(9));    
    betai = theta(na+1+2:na+1+7);
    ai = theta(8:13)-tempsig*betai;     
    ama2 = alp2grid-a2;
    sigu2 = c^2+ama2.^2*sigv^2/beta2^2;
    alp0 = a0+ama2*beta0/beta2;
    alp1 = a1+ama2*beta1/beta2;    
    gam = b-ama2/beta2;
    rho2 = ama2.^2*sigv^2/beta2^2./sigu2;
    tempalp0 = -.5*invVtheta(1,1)*alp0.^2;
    tempalp1 = -.5*invVtheta(2,2)*alp1.^2;
    tempalp2 = -.5*invVtheta(3,3)*alp2grid.^2;   
    tempalpi = 0;
    for ii = 1:6
        alpi = ai(ii)+ama1*betai(ii)/beta1;
        tempalpi = tempalpi -.5*invVtheta(7+ii,7+ii)*alpi.^2;
    end   
    tempJ = log(sigv/abs(beta2)) - .5*log(sigu2);
    detSig = (1-rho2).*sigu2*sigv^2;
    tempSig = -.5*(nu0+3)*log(detSig) -.5./detSig.*(sigv^2 + sigu2);
    if priorflag == 1; 
        palp2 = exp(tempalp0 + tempalp1 + tempalp2 + tempalpi + tempSig + tempJ);
    else        
        ind1 = abs(alp1)<=abs(gamub*beta1);        
        ind2 = abs(alp2grid)<=abs(gamub*beta2);   
        palp2 = exp(tempalp0 + tempalp1 + tempalp2 + tempalpi + tempSig + tempJ).*ind2.*ind1;             
    end
    spalp2 = palp2 + spalp2;  
end
palp2hat = spalp2/M;
kalp2 = (alp2grid(3)-alp2grid(2))*sum(palp2hat);
palp2hat = palp2hat/kalp2;
Ealp2 = alp2grid'*palp2hat/sum(palp2hat);
Ealp22 = (alp2grid.^2)'*palp2hat/sum(palp2hat);
alp2std = sqrt(Ealp22-Ealp2^2);

figure;
subplot(1,2,1); plot(alp1grid,palp1hat); title('\alpha_1'); box off;
subplot(1,2,2); plot(alp2grid,palp2hat); title('\alpha_2'); box off;
set(gcf,'Position',[100 100 1000 400]);

